from os.path import exists, join
from os import makedirs
import ipdb
import numpy as np
import matplotlib.pyplot as plt
from joblib import dump, load

from sklearn.cluster import DBSCAN
from sklearn.model_selection import GridSearchCV
from sklearn.datasets import make_classification

from config import MIN_RESP_RATE, MAX_RESP_RATE

TUNER_TYPE = "gridsearchcv"
DEBUG = False

class DBSCANClass():
    ''' Class to handle the linear discriminant analysis under gridsearch, must
    have unilateral data classification'''
    def __init__(self, marker=1):
        # model parameters
        if DEBUG:
            self.param_grid = [
                {
                    "eps": [0.5],
                    "leaf_size": [10],
                },
            ]
            self.verbose = 1
        else:
            self.param_grid = [
                {
                    "eps": [0.5],
                    "leaf_size": [30],
                    "min_samples": [30],
                },
            ]
            self.verbose = 0
        self.scoring = ['normalized_mutual_info_score']

        self.eval_csv = './evaluations/dbscan.csv'
        self.directory = f'/data/rqchia/ARIA/dbscan_{marker}'

    def build(self):
        self.model = DBSCAN()
        return self.model

    def save_model(self, filename='dbscan_model.joblib'):
        # save the model to disk
        if not exists(self.directory):
            makedirs(self.directory)
        fileout = join(self.directory, filename)
        with open(filename, 'wb') as mfile:
            dump(self.model, mfile)

    def load_model(self, filename='dbscan_model.joblib'):
        # load the model from disk
        # self.model = load(filename)
        filein = join(self.directory, filename)
        with open(filein, 'rb') as mfile:
            self.model = load(mfile)

def main():
    # test classification dataset
    print("Making data")
    n_features = 10
    x, y = make_classification(n_samples=1000, n_features=n_features,
                               n_informative=6, n_redundant=4, random_state=1,
                               n_classes=3)
    print('classes: ', np.unique(y))

    # define model
    knn = KNNClass()
    model = knn.build()
    

    # fit model
    print("Fitting data")
    model.fit(x, y)

    # new data
    row = np.random.rand(1, n_features) + np.random.randint(low=-4, high=4,
                                                            size=(1,n_features)
                                                           )
    print("row: ", row)
    print("Predicting")
    yhat = model.predict(row)

    print('Predicted Class: %d' % yhat)

    plt.plot(x[y==0,0], x[y==0,1], 'b.')
    plt.plot(x[y==1,0], x[y==1,1], 'r.')
    plt.plot(x[y==2,0], x[y==2,1], 'k.')
    plt.plot(row[0,0], row[0, 1], 'y^')
    plt.show()

if __name__ == '__main__':
    main()
