from os.path import join, exists
from os import makedirs
import ipdb
import numpy as np
import matplotlib.pyplot as plt
from joblib import dump, load
from os.path import join

from sklearn.svm import SVR
from sklearn.model_selection import GridSearchCV
from sklearn.datasets import make_classification

TUNER_TYPE = "gridsearchcv"
DEBUG = False

class SVRClass():
    ''' Class to handle the linear discriminant analysis under gridsearch, must
    have unilateral data classification'''
    def __init__(self, marker=1, directory=None):
        # model parameters, for SVC
        if DEBUG:
            self.param_grid = [
                {
                    "degree": [1, 2],
                },
            ]
            self.verbose = 1
        else:
            self.param_grid = [
                {
                    "kernel": ['rbf', 'poly'],
                    "degree": [1, 2],
                },
            ]
            self.verbose = 0
        self.scoring = ['neg_mean_absolute_error',
                        'neg_root_mean_squared_error',
                       ]
        if directory is None:
            self.directory = f'/data/rqchia/ARIA/svr_{marker}'
        else:
            self.directory = directory

    def build(self):
        # self.model = SVR(verbose=self.verbose, degree=4)
        self.model = GridSearchCV(
            SVR(verbose=self.verbose),
            param_grid=self.param_grid,
            scoring=self.scoring,
            cv=3,
            refit='neg_mean_absolute_error'
        )
        return self.model

    def save_model(self, filename='svr_model.joblib'):
        # save the model to disk
        if not exists(self.directory): makedirs(self.directory)
        fileout = join(self.directory, filename)
        with open(fileout, 'wb') as mfile:
            dump(self.model, mfile)

    def load_model(self, filename='svr_model.joblib'):
        # load the model from disk
        # self.model = load(filename)
        filein = join(self.directory, filename)
        with open(filein, 'rb') as mfile:
            self.model = load(mfile)

def main():
    # test classification dataset
    print("Making data")
    n_features = 10
    x, y = make_classification(n_samples=1000, n_features=n_features,
                               n_informative=6, n_redundant=4, random_state=1,
                               n_classes=3)
    print('classes: ', np.unique(y))

    # define model
    svr = SVRClass()
    model = svr.build()

    # fit model
    print("Fitting data")
    model.fit(x, y)

    # new data
    row = np.random.rand(1, n_features) + np.random.randint(low=-4, high=4,
                                                            size=(1,n_features)
                                                           )
    print("row: ", row)
    print("Predicting")
    yhat = model.predict(row)

    print('Predicted Class: %d' % yhat)

    plt.plot(x[y==0,0], x[y==0,1], 'b.')
    plt.plot(x[y==1,0], x[y==1,1], 'r.')
    plt.plot(x[y==2,0], x[y==2,1], 'k.')
    plt.plot(row[0,0], row[0, 1], 'y^')
    plt.show()

if __name__ == '__main__':
    main()
