import numpy as np
import xgboost as xgb
import pandas as pd

from os.path import join, exists
from os import makedirs
import ipdb
import matplotlib.pyplot as plt
from sklearn.metrics import mean_absolute_error, f1_score
from sklearn.datasets import make_classification
from sklearn.model_selection import RandomizedSearchCV

class XGBoostClass():
    ''' Class to handle the linear discriminant analysis under gridsearch, must
    have unilateral data classification'''
    def __init__(self, marker=1, directory=None):
        self.params = {
            'max_depth': [3, 5, 6, 10, 15, 20],
            'learning_rate': [0.01, 0.1, 0.2, 0.3],
            'subsample': np.arange(0.5, 1.0, 0.1),
            'colsample_bytree': np.arange(0.4, 1.0, 0.1),
            'colsample_bylevel': np.arange(0.4, 1.0, 0.1),
            'n_estimators': [100, 500, 1000]
        }
        self.scoring = ['neg_mean_absolute_error',
                        'neg_root_mean_squared_error',
                        'r2']

        self.seed = 1234
        self.n_iter = 20
        self.eval_metric = mean_absolute_error
        if directory is None:
            self.directory = f'/data/rqchia/ARIA/xgboost_{marker}'
        else:
            self.directory = directory
        self.eval_csv = './evaluations/xgboost.csv'
        self.mdl_type = 'regressor'
        self.is_search = True

    def build(self):
        if self.mdl_type == 'regressor':
            self.eval_metric = mean_absolute_error
            estimator = xgb.XGBRegressor(seed=self.seed, device='cuda',
                                         tree_method='hist')
            scoring = 'neg_mean_absolute_error'
        elif self.mdl_type == 'classifier':
            self.eval_metric = f1_score
            estimator = xgb.XGBClassifier(seed=self.seed, device='cuda',
                                          tree_method='hist')
            scoring = 'f1'

        if self.is_search:
            self.model = RandomizedSearchCV(
                estimator=estimator,
                param_distributions=self.params,
                scoring=scoring,
                n_iter=self.n_iter,
                verbose=1)
        else:
            self.model = model
        return self.model

    def save_model(self, filename='xgb_model.model'):
        # save the model to disk
        if not exists(self.directory): makedirs(self.directory)
        fileout = join(self.directory, filename)
        if self.is_search:
            self.model.best_estimator_.save_model(fileout)
        else:
            self.model.save_model(fileout)

    def load_model(self, filename='xgb_model.model'):
        # load the model from disk
        mdl = mdl_cls.model
        # self.model = load(filename)
        filein = join(self.directory, filename)
        self.model = xgb.Booster()
        self.model.load_model(filein)

def main():
    # test classification dataset
    print("Making data")
    n_features = 10
    x, y = make_classification(n_samples=1000, n_features=n_features,
                               n_informative=6, n_redundant=4, random_state=1,
                               n_classes=3)
    print('classes: ', np.unique(y))

    # define model
    xgb_cls = XGBoostClass()
    xgb_cls.build()

    # fit model
    print("Fitting data")
    # dtrain = xgb_cls.DMatrix(data=x, label=y)
    xgb_cls.model.fit(x, y)

    # new data
    row = np.random.rand(1, n_features) + np.random.randint(low=-4, high=4,
                                                            size=(1,n_features)
                                                           )
    print("row: ", row)
    print("Predicting")
    yhat = xgb_cls.model.predict(row)

    print('Predicted Class: %d' % yhat)

    plt.plot(x[y==0,0], x[y==0,1], 'b.')
    plt.plot(x[y==1,0], x[y==1,1], 'r.')
    plt.plot(x[y==2,0], x[y==2,1], 'k.')
    plt.plot(row[0,0], row[0, 1], 'y^')
    plt.show()

if __name__ == '__main__':
    main()
